/*
XLiFE++ is an extended library of finite elements written in C++
    Copyright (C) 2014  Lunéville, Eric; Kielbasiewicz, Nicolas; Lafranche, Yvon; Nguyen, Manh-Ha; Chambeyron, Colin

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

/*!
  \file DomainMap.hpp
  \author E. Lunéville
  \since 04 apr 2012
  \date 04 apr 2012

  \brief Definition of xlifepp::DomainMap class and functionalities
 */

#ifndef DOMAIN_MAP_HPP
#define DOMAIN_MAP_HPP

#include "config.h"
#include "utils.h"
#include "GeomDomain.hpp"

namespace xlifepp
{

/*!
    \class DomainMap
    handles map (declared as Function) between two GeomDomain's.
    Only one map between domains may be defined !!!
 */
class DomainMap
{
  protected:
    const GeomDomain* dom1_p; //!< start domain
    const GeomDomain* dom2_p; //!< end domain
    Function map1to2_;          //!< map function from domain1 to domain2
    DomainMap(){}               //!< shadow default constructor

  public:
    DomainMap(const GeomDomain&, const GeomDomain&, const Function&);  //!< explicit constructor
    ~DomainMap(); //!< destructor

    //list of all DomainMap
    static std::vector<DomainMap *> theDomainMaps; //!< list of existing maps
    static void clearGlobalVector();               //!< delete all map objects

    //accessors
    const GeomDomain& dom1() const //! returns start domain
    {return *dom1_p;}
    const GeomDomain& dom2() const //! returns end domain
    {return *dom2_p;}
    const Function& map1to2() const //! returns map function
    {return map1to2_;}
};

void defineMap(const GeomDomain&, const GeomDomain&, const Function&); //!< define a map between 2 geomdomains
const Function* findMap(const GeomDomain&, const GeomDomain&);         //!< find map between 2 geomdomains


} // end of namespace xlifepp

#endif // DOMAIN_MAP_HPP
