/*
XLiFE++ is an extended library of finite elements written in C++
    Copyright (C) 2014  Lunville, Eric; Kielbasiewicz, Nicolas; Lafranche, Yvon; Nguyen, Manh-Ha; Chambeyron, Colin

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

/*!
  \file randomGenerators.hpp
  \author E. Lunville
  \since 12 jul 2016
  \date 12 jul 2016

  \brief Definition of some basic random generators
*/

#ifndef RANDOM_GENERATORS_HPP
#define RANDOM_GENERATORS_HPP

#include "config.h"
#include <iostream>

namespace xlifepp
{
 enum GaussianGenerator{_BoxMullerGenerator,_MarsagliaGenerator};

 real_t normalBoxMuller(real_t mu=0., real_t sigma=1.); //!< normal distribution (mu,sigma) using Box Muller method
 real_t normalMarsaglia(real_t mu=0., real_t sigma=1.); //!< normal distribution (mu,sigma) using Marsaglia method
 real_t normalDistribution(real_t mu=0., real_t sigma=1.,GaussianGenerator gg=_MarsagliaGenerator); //!< return a sample from normal distribution (mu,sigma)
 void normalDistribution(real_t* mat, real_t mu, real_t sigma, number_t n=1, number_t m=1);     //!< compute a Gaussian matrix normal distribution (mu,sigma)
 void normalDistribution(real_t* mat, number_t n=1, number_t m=1);                              //!< compute a Gaussian matrix normal distribution (0,1)
 void normalDistribution(complex_t* mat, real_t mu, real_t sigma, number_t n=1, number_t m=1);  //!< compute a complex Gaussian matrix normal distribution (mu,sigma)
 void normalDistribution(complex_t* mat, number_t n=1, number_t m=1);                           //!< compute a complex Gaussian matrix normal distribution (0,1)

} // end of namespace xlifepp

#endif /* GAUSS_FORMULAE_HPP */

