/*
XLiFE++ is an extended library of finite elements written in C++
    Copyright (C) 2014  Lunéville, Eric; Kielbasiewicz, Nicolas; Lafranche, Yvon; Nguyen, Manh-Ha; Chambeyron, Colin

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

/*!
  \file Reals.hpp
  \authors N. Kielbasiewicz
  \since 10 sep 2015
  \date 11 sep 2015

  \brief xlifepp::Reals class inheriting from std::vector<real_t> is a simple interface to a vector of numbers
 */

#ifndef REALS_HPP
#define REALS_HPP

#include "config.h"
#include "Vector.hpp"

namespace xlifepp
{

/*!
  \class Reals
  a simple interface to Vector<real_t>
*/
class Reals : public Vector<real_t>
{
  public:
    //! access to n-th number (const)
    const real_t& operator()(number_t n) const
    {
      if(n<1 || n>size()) {std::cout<<"Reals : out of bounds"; exit(-1);}
      return at(n-1);
    }
    //! access to n-th number (non const)
    real_t& operator()(number_t n)
    {
      if(n<1 || n>size()) {std::cout<<"Reals : out of bounds"; exit(-1);}
      return at(n-1);
    }

    //! constructor from size and const number
    explicit Reals(const number_t& d=0, const real_t& r=0) : Vector<real_t>(d,r) {}
    //! constructor from size and const number
    explicit Reals(const int& d, const real_t& r=0) : Vector<real_t>(d,r) {}
    //! constructor from std::vector<real_t>
    Reals(const std::vector<real_t>& rv) : Vector<real_t>(rv) {}
    //! constructor from 2 real_t objects
    Reals(const real_t& r1, const real_t& r2)
      : Vector<real_t>(2)
    {
      at(0)=r1;
      at(1)=r2;
    }
    //! constructor from 3 real_t objects
    Reals(const real_t& r1, const real_t& r2, const real_t& r3)
      : Vector<real_t>(3)
    {
      at(0)=r1;
      at(1)=r2;
      at(2)=r3;
    }
    //! constructor from 4 real_t objects
    Reals(const real_t& r1, const real_t& r2, const real_t& r3, const real_t& r4)
      : Vector<real_t>(4)
    {
      at(0)=r1;
      at(1)=r2;
      at(2)=r3;
      at(3)=r4;
    }
    //! constructor from 5 real_t objects
    Reals(const real_t& r1, const real_t& r2, const real_t& r3, const real_t& r4, const real_t& r5)
      : Vector<real_t>(5)
    {
      at(0)=r1;
      at(1)=r2;
      at(2)=r3;
      at(3)=r4;
      at(4)=r5;
    }
    //! constructor from 6 real_t objects
    Reals(const real_t& r1, const real_t& r2, const real_t& r3, const real_t& r4, const real_t& r5, const real_t& r6)
      : Vector<real_t>(6)
    {
      at(0)=r1;
      at(1)=r2;
      at(2)=r3;
      at(3)=r4;
      at(4)=r5;
      at(5)=r6;
    }
    //! constructor from 7 real_t objects
    Reals(const real_t& r1, const real_t& r2, const real_t& r3, const real_t& r4, const real_t& r5, const real_t& r6,
            const real_t& r7)
      : Vector<real_t>(7)
    {
      at(0)=r1;
      at(1)=r2;
      at(2)=r3;
      at(3)=r4;
      at(4)=r5;
      at(5)=r6;
      at(6)=r7;
    }
    //! constructor from 8 real_t objects
    Reals(const real_t& r1, const real_t& r2, const real_t& r3, const real_t& r4, const real_t& r5, const real_t& r6,
            const real_t& r7, const real_t& r8)
      : Vector<real_t>(8)
    {
      at(0)=r1;
      at(1)=r2;
      at(2)=r3;
      at(3)=r4;
      at(4)=r5;
      at(5)=r6;
      at(6)=r7;
      at(7)=r8;
    }
    //! constructor from 9 real_t objects
    Reals(const real_t& r1, const real_t& r2, const real_t& r3, const real_t& r4, const real_t& r5, const real_t& r6,
            const real_t& r7, const real_t& r8, const real_t& r9)
      : Vector<real_t>(9)
    {
      at(0)=r1;
      at(1)=r2;
      at(2)=r3;
      at(3)=r4;
      at(4)=r5;
      at(5)=r6;
      at(6)=r7;
      at(7)=r8;
      at(8)=r9;
    }
    //! constructor from 10 real_t objects
    Reals(const real_t& r1, const real_t& r2, const real_t& r3, const real_t& r4, const real_t& r5, const real_t& r6,
            const real_t& r7, const real_t& r8, const real_t& r9, const real_t& r10)
      : Vector<real_t>(10)
    {
      at(0)=r1;
      at(1)=r2;
      at(2)=r3;
      at(3)=r4;
      at(4)=r5;
      at(5)=r6;
      at(6)=r7;
      at(7)=r8;
      at(8)=r9;
      at(9)=r10;
    }

    void print(std::ostream& out) const
    {
        std::vector<real_t>::const_iterator its=begin();
        number_t i=1;
        for(;its!=end();++its, ++i) out<<" real "<<i<<" : "<<(*its)<<eol;
    }

    void print(PrintStream& os) const {print(os.currentStream());}

    friend std::ostream& operator<<(std::ostream& out, const Reals& rs)
    {rs.print(out); return out;}
};

inline Reals& operator<<(Reals& rs, const real_t& r)
{
  rs.push_back(r);
  return rs;
}

} // end of namespace xlifepp;

#endif /* REALS_HPP */
