# FindCHOLMOD is aimed to be a submodule of FindUMFPACK
# It does not work independently !
# Cholmod lib usually requires linking to a blas library.
# It is up to the user of this module to find a BLAS and link to it.
#
# module inputs
#     CHOLMOD_INCLUDE_DIR = specific directory where to find cholmod.h
#     CHOLMOD_LIB_DIR = specific directory where to find cholmod library
# module outputs
#     CHOLMOD_INCLUDES = cholmod.h and other includes with full paths
#     CHOLMOD_LIBRARIES = cholmod library and other libraries with full paths

if (CHOLMOD_INCLUDES AND CHOLMOD_LIBRARIES)
  set(CHOLMOD_FIND_QUIETLY TRUE)
endif(CHOLMOD_INCLUDES AND CHOLMOD_LIBRARIES)

# find dependencies
find_package(AMD)
find_package(CAMD)
find_package(COLAMD)
find_package(CCOLAMD)
find_package(METIS)

# Check for header files
find_path(CHOLMOD_INCLUDE NAMES cholmod.h PATHS ${CHOLMOD_INCLUDE_DIR} $ENV{CHOLMOD_INCLUDE_DIR} ${SUITESPARSE_DIR} $ENV{SUITESPARSE_DIR} PATH_SUFFIXES suitesparse ufsparse include Include NO_DEFAULT_PATH)
find_path(CHOLMOD_INCLUDE NAMES cholmod.h PATHS ${INCLUDE_INSTALL_DIR} PATH_SUFFIXES suitesparse ufsparse include Include)
set(CHOLMOD_INCLUDES)
if (CHOLMOD_INCLUDE)
  get_filename_component(CHOLMOD_INCLUDE ${CHOLMOD_INCLUDE} REALPATH)
  message(STATUS "Found cholmod.h : ${CHOLMOD_INCLUDE}")
  list(APPEND CHOLMOD_INCLUDES  ${CHOLMOD_INCLUDE})
endif()
if (AMD_INCLUDE)
  set(amdFlagIndex "-1")
  if (CHOLMOD_INCLUDES)
    list(FIND CHOLMOD_INCLUDES ${AMD_INCLUDE} amdFlagIndex)
  endif()
  if (amdFlagIndex STREQUAL "-1")
    list(APPEND CHOLMOD_INCLUDES ${AMD_INCLUDE})
  endif()
endif()
if (CAMD_INCLUDE)
  set(camdFlagIndex "-1")
  if (CHOLMOD_INCLUDES)
    list(FIND CHOLMOD_INCLUDES ${CAMD_INCLUDE} camdFlagIndex)
  endif()
  if (camdFlagIndex STREQUAL "-1")
    list(APPEND CHOLMOD_INCLUDES ${CAMD_INCLUDE})
  endif()
endif()
if (COLAMD_INCLUDE)
  set(colamdFlagIndex "-1")
  if (CHOLMOD_INCLUDES)
    list(FIND CHOLMOD_INCLUDES ${COLAMD_INCLUDE} colamdFlagIndex)
  endif()
  if (colamdFlagIndex STREQUAL "-1")
    list(APPEND CHOLMOD_INCLUDES ${COLAMD_INCLUDE})
  endif()
endif()
if (CCOLAMD_INCLUDE)
  set(ccolamdFlagIndex "-1")
  if (CHOLMOD_INCLUDES)
    list(FIND CHOLMOD_INCLUDES ${CCOLAMD_INCLUDE} ccolamdFlagIndex)
  endif()
  if (ccolamdFlagIndex STREQUAL "-1")
    list(APPEND CHOLMOD_INCLUDES ${CCOLAMD_INCLUDE})
  endif()
endif()
if (METIS_INCLUDE)
  set(metisFlagIndex "-1")
  if (CHOLMOD_INCLUDES)
    list(FIND CHOLMOD_INCLUDES ${METIS_INCLUDE} metisFlagIndex)
  endif()
  if (metisFlagIndex STREQUAL "-1")
    list(APPEND CHOLMOD_INCLUDES ${METIS_INCLUDE})
  endif()
endif()
mark_as_advanced(CHOLMOD_INCLUDE CHOLMOD_INCLUDES)

# Check for CHOLMOD libraries
find_library(CHOLMOD_LIBRARY cholmod PATHS ${CHOLMOD_LIB_DIR} $ENV{CHOLMOD_LIB_DIR} ${SUITESPARSE_DIR} $ENV{SUITESPARSE_DIR} PATH_SUFFIXES Lib lib NO_DEFAULT_PATH)
find_library(CHOLMOD_LIBRARY cholmod PATHS ${LIB_INSTALL_DIR} PATH_SUFFIXES Lib lib)
set(CHOLMOD_LIBRARIES)
if (CHOLMOD_LIBRARY)
  get_filename_component(CHOLMOD_LIBRARY ${CHOLMOD_LIBRARY} REALPATH)
  message(STATUS "Found CHOLMOD : ${CHOLMOD_LIBRARY}")
  set(CHOLMOD_LIBRARIES ${CHOLMOD_LIBRARIES} ${CHOLMOD_LIBRARY})
endif()
if (AMD_LIBRARY)
  set(CHOLMOD_LIBRARIES ${CHOLMOD_LIBRARIES} ${AMD_LIBRARY})
endif()
if (CAMD_LIBRARY)
  set(CHOLMOD_LIBRARIES ${CHOLMOD_LIBRARIES} ${CAMD_LIBRARY})
endif()
if (COLAMD_LIBRARY)
  set(CHOLMOD_LIBRARIES ${CHOLMOD_LIBRARIES} ${COLAMD_LIBRARY})
endif()
if (CCOLAMD_LIBRARY)
  set(CHOLMOD_LIBRARIES ${CHOLMOD_LIBRARIES} ${CCOLAMD_LIBRARY})
endif()
if (METIS_LIBRARY)
  set(CHOLMOD_LIBRARIES ${CHOLMOD_LIBRARIES} ${METIS_LIBRARY})
endif()
mark_as_advanced(CHOLMOD_LIBRARY CHOLMOD_LIBRARIES)

# Standard package handling
include(FindPackageHandleStandardArgs)
find_package_handle_standard_args(CHOLMOD DEFAULT_MSG CHOLMOD_LIBRARIES CHOLMOD_INCLUDES)
