set(CMAKE_MODULE_PATH ${CMAKE_MODULE_PATH} "${CMAKE_SOURCE_DIR}/cmake/modules")

######################## options definition #########################

# default config
set(ARPACK_OPTION_DEFAULT OFF)
set(UMFPACK_OPTION_DEFAULT OFF)
set(OMP_OPTION_DEFAULT ON)
set(EIGEN_OPTION_DEFAULT ON)
set(AMOS_OPTION_DEFAULT ON)
set(GFORTRAN_OPTION_DEFAULT OFF)
set(BLAS_LIB_DEFAULT " ")
set(LAPACK_LIB_DEFAULT " ")
set(ARPACK_LIB_DEFAULT " ")
set(SUITESPARSE_HOME_DIR_DEFAULT " ")
if (NOT XLIFEPP_FROM_SRCS)
  message(STATUS "Loading configuration of binary distribution")
  include(${CMAKE_SOURCE_DIR}/cmake/getBinaryConfig.cmake)
endif()

option(XLIFEPP_ENABLE_ARPACK "Enable ARPACK (ON, OFF)" ${ARPACK_OPTION_DEFAULT})
option(XLIFEPP_ENABLE_UMFPACK "Enable UmfPack (ON, OFF)" ${UMFPACK_OPTION_DEFAULT})
option(XLIFEPP_ENABLE_OMP "Enable OpenMP (ON, OFF)" ${OMP_OPTION_DEFAULT})
option(XLIFEPP_ENABLE_EIGEN "Enable Eigen (ON, OFF)" ${EIGEN_OPTION_DEFAULT})
option(XLIFEPP_ENABLE_AMOS "Enable Amos (ON, OFF)" ${AMOS_OPTION_DEFAULT})
option(XLIFEPP_ENABLE_GFORTRAN "Enable link with gfortran library (ON, OFF)" ${GFORTRAN_OPTION_DEFAULT})
mark_as_advanced(XLIFEPP_ENABLE_GFORTRAN)

set(XLIFEPP_DEPS DEFAULT CACHE STRING "Global variable to activate/deactivate every external dependencies. Possible values are ENABLE_ALL, DISABLE_ALL or DEFAULT")

if (${XLIFEPP_DEPS} STREQUAL ENABLE_ALL)
  set(XLIFEPP_ENABLE_ARPACK ON CACHE BOOL "Enable ARPACK (ON, OFF)" FORCE)
  set(XLIFEPP_ENABLE_UMFPACK ON CACHE BOOL "Enable UmfPack (ON, OFF)" FORCE)
  set(XLIFEPP_ENABLE_OMP ON CACHE BOOL "Enable OpenMP (ON, OFF)" FORCE)
  set(XLIFEPP_ENABLE_EIGEN ON CACHE BOOL "Enable Eigen (ON, OFF)" FORCE)
  set(XLIFEPP_ENABLE_AMOS ON CACHE BOOL "Enable Amos (ON, OFF)" FORCE)
elseif(${XLIFEPP_DEPS} STREQUAL DISABLE_ALL)
  set(XLIFEPP_ENABLE_ARPACK OFF CACHE BOOL "Enable ARPACK (ON, OFF)" FORCE)
  set(XLIFEPP_ENABLE_UMFPACK OFF CACHE BOOL "Enable UmfPack (ON, OFF)" FORCE)
  set(XLIFEPP_ENABLE_OMP OFF CACHE BOOL "Enable OpenMP (ON, OFF)" FORCE)
  set(XLIFEPP_ENABLE_EIGEN OFF CACHE BOOL "Enable Eigen (ON, OFF)" FORCE)
  set(XLIFEPP_ENABLE_AMOS OFF CACHE BOOL "Enable Amos (ON, OFF)" FORCE)
endif()

set(FORTRAN_ENABLED_FALSE)

######################### utility functions ##########################

# definition of a macro (MACRO) from another one (CACHE_MACRO) that can be already in cache,
# and a default value if CACHE_MACRO exists in cache and not empty
function(xlifeppSetDefaultValue MACRO CACHE_MACRO DEFAULTVALUE)
  set(MACRO_EXISTS -1)
  get_cmake_property(cachelist CACHE_VARIABLES)
  list(FIND cachelist ${CACHE_MACRO} MACRO_EXISTS)
  if (NOT ${MACRO_EXISTS} STREQUAL "-1")
    if (NOT ${${CACHE_MACRO}} STREQUAL " ")
      set(${MACRO} ${${CACHE_MACRO}} PARENT_SCOPE)
    else()
      set(${MACRO} ${DEFAULTVALUE} PARENT_SCOPE)
    endif()
  else()
    set(${MACRO} ${DEFAULTVALUE} PARENT_SCOPE)
  endif()
  unset(cachelist CACHE)
endfunction()

##### OMP -> XLIFEPP_WITH_OMP or XLIFEPP_WITHOUT_OMP

set(OMP XLIFEPP_WITHOUT_OMP)
if (XLIFEPP_ENABLE_OMP)
  find_package(OpenMP)
  if (OPENMP_FOUND)
    set(OMP XLIFEPP_WITH_OMP)
    set(CMAKE_CXX_FLAGS "${CMAKE_CXX_FLAGS} ${OpenMP_CXX_FLAGS}")
    mark_as_advanced(CLEAR OpenMP_CXX_FLAGS)
    if(OpenMP_EXE_LINKER_FLAGS)
      set(CMAKE_EXE_LINKER_FLAGS "${CMAKE_EXE_LINKER_FLAGS} ${OpenMP_EXE_LINKER_FLAGS}")
    endif(OpenMP_EXE_LINKER_FLAGS)
    message(STATUS "OpenMP is used")
  else()
    message("####################################")
    message(STATUS "###### OpenMP is not supported ! It is not used")
    message("####################################")
  endif()
else()
  message(STATUS "OpenMP is not used")
endif()

###################### LAPACK/BLAS management ########################
set(XLIFEPP_ENABLE_LAPACK FALSE)
set(XLIFEPP_ENABLE_BLAS FALSE)

if (XLIFEPP_ENABLE_ARPACK OR XLIFEPP_ENABLE_UMFPACK)
  set(XLIFEPP_ENABLE_LAPACK TRUE)
  set(XLIFEPP_ENABLE_BLAS TRUE)
endif()

  ##### Fifth macro : BLAS -> XLIFEPP_WITH_BLAS or XLIFEPP_WITHOUT_BLAS
  #    cache option : XLIFEPP_BLAS_LIB_DIR
  #####
  set(XLIFEPP_BLAS_LIB_DIR " " CACHE PATH "Path to BLAS lib directory")
  set(XLIFEPP_BLAS_LIB ${BLAS_LIB_DEFAULT} CACHE FILE "BLAS lib")
  set(BLAS XLIFEPP_WITHOUT_BLAS)
  if (XLIFEPP_ENABLE_BLAS)
    if (XLIFEPP_BLAS_LIB STREQUAL " ")
      find_blas(${XLIFEPP_BLAS_LIB_DIR} BLAS_FOUND XLIFEPP_BLAS_LIB)
    else()
      if (EXISTS ${XLIFEPP_BLAS_LIB})
        set(BLAS_FOUND TRUE)
		    get_filename_component(XLIFEPP_BLAS_LIB_DIR ${XLIFEPP_BLAS_LIB} REALPATH)
      else()
        set(BLAS_FOUND FALSE)
      endif()
    endif()
    if (BLAS_FOUND)
      set(BLAS XLIFEPP_WITH_BLAS)
      message(STATUS "BLAS is used")
    else()
      set(BLAS XLIFEPP_WITHOUT_BLAS)
      message("####################################")
      message("##### BLAS is not found ! It is not used")
      message("####################################")
    endif()
  else()
    message(STATUS "BLAS is not used")
  endif()
  
  ##### Sixth macro : LAPACK -> XLIFEPP_WITH_LAPACK or XLIFEPP_WITHOUT_LAPACK
  #    cache option : XLIFEPP_LAPACK_LIB_DIR
  #####
  set(XLIFEPP_LAPACK_LIB_DIR " " CACHE PATH "Path to LAPACK lib directory")
  set(XLIFEPP_LAPACK_LIB ${LAPACK_LIB_DEFAULT} CACHE FILE "LAPACK lib")
  set(LAPACK XLIFEPP_WITHOUT_LAPACK)
  if (XLIFEPP_ENABLE_LAPACK)
    if (XLIFEPP_LAPACK_LIB STREQUAL " ")
      find_lapack(${XLIFEPP_LAPACK_LIB_DIR} LAPACK_FOUND XLIFEPP_LAPACK_LIB)
    else()
      if (EXISTS ${XLIFEPP_LAPACK_LIB})
        set(LAPACK_FOUND TRUE)
		    get_filename_component(XLIFEPP_LAPACK_LIB_DIR ${XLIFEPP_LAPACK_LIB} REALPATH)
      else()
        set(LAPACK_FOUND FALSE)
      endif()
    endif()
    if (LAPACK_FOUND)
      set(LAPACK XLIFEPP_WITH_LAPACK)
      message(STATUS "Lapack is used")
    else()
      set(LAPACK XLIFEPP_WITHOUT_LAPACK)
      message("####################################")
      message("##### Lapack is not found ! It is not used")
      message("####################################")
    endif()
  else()
    message(STATUS "LAPACK is not used")
  endif()
#endif()

mark_as_advanced(XLIFEPP_BLAS_LIB)
mark_as_advanced(XLIFEPP_LAPACK_LIB)

##### Seventh macro : ARPACK -> XLIFEPP_WITH_ARPACK or XLIFEPP_WITHOUT_ARPACK
#     ----------------------- LAPACK and BLAS required --------------------------
# cache option : ARPACK_LIB_DIR, XLIFEPP_ENABLE_ARPACK
# behavior : if XLIFEPP_ARPACK_LIB_DIR is set, XLIFEPP_ENABLE_ARPACK=ON
#            if XLIFEPP_ENABLE_ARPACK=OFF, ARPACK_LIB_DIR is removed from cache
# Checking Arpack++ package
set(XLIFEPP_ARPACKPP_INCLUDE_DIR ${CMAKE_SOURCE_DIR}/ext/ARPACK++/include CACHE PATH "Path to Arpack++ include directory")
set(XLIFEPP_ARPACK_LIB_DIR " " CACHE PATH "Path to arpack directory")
set(XLIFEPP_ARPACK_LIB ${ARPACK_LIB_DEFAULT} CACHE FILE "ARPACK lib")
set(ARPACK XLIFEPP_WITHOUT_ARPACK)
set(AUTOBUILDARPACK FALSE)
if (XLIFEPP_ENABLE_ARPACK)
  set(ARPACK_EXISTS -1)
  include_directories(${XLIFEPP_ARPACKPP_INCLUDE_DIR})
  if (XLIFEPP_ARPACK_LIB STREQUAL " ")
    find_arpack(${XLIFEPP_ARPACK_LIB_DIR} ARPACK_FOUND XLIFEPP_ARPACK_LIB)
  endif()
  if (NOT XLIFEPP_ARPACK_LIB STREQUAL " ")
    if (NOT LAPACK_FOUND)
      message("####################################")
      message("###### LAPACK and/or BLAS not found ! ARPACK is not used")
      message("####################################")
    elseif (NOT BLAS_FOUND)
      message("####################################")
      message("###### LAPACK and/or BLAS not found ! ARPACK is not used")
      message("####################################")
    else()
      get_filename_component(XLIFEPP_ARPACK_LIB_DIR ${XLIFEPP_ARPACK_LIB} REALPATH)
      set(ARPACK XLIFEPP_WITH_ARPACK)
      message(STATUS "Arpack is used")
      if (CMAKE_COMPILER_IS_GNUCXX)
        set(GCC_COVERAGE_COMPILE_FLAGS "-fpermissive")
        set(permissiveFlagIndex "-1")
        if (CMAKE_CXX_FLAGS)
          string(FIND ${CMAKE_CXX_FLAGS} ${GCC_COVERAGE_COMPILE_FLAGS} permissiveFlagIndex)
        endif()
        if (permissiveFlagIndex STREQUAL "-1")
          set(CMAKE_CXX_FLAGS "${CMAKE_CXX_FLAGS} ${GCC_COVERAGE_COMPILE_FLAGS}" CACHE STRING "compile flags" FORCE)
        endif()
      endif()
    endif()
  else()
    if (XLIFEPP_ARPACK_LIB_DIR STREQUAL " ")
      enable_language(Fortran)
      if (CMAKE_SYSTEM_NAME MATCHES "Darwin")
        cmake_policy(SET CMP0042 NEW) # needed by a cmake warning because of compilation of arpack library
      endif()
      set(FORTRAN_ENABLED TRUE)
      set(XLIFEPP_ENABLE_GFORTRAN TRUE)
      set(XLIFEPP_ARPACK_LIB_DIR "${CMAKE_SOURCE_DIR}/libext/${XLIFEPP_ARCH}/${XLIFEPP_CXX_COMPILER}/${CMAKE_BUILD_TYPE}")
      message(STATUS "Arpack will be compiled and used")
      add_subdirectory(ext/ARPACK)
      set(ARPACK XLIFEPP_WITH_ARPACK)
      set(ARPACK_FOUND TRUE)
      set(XLIFEPP_ARPACK_LIB ${XLIFEPP_ARPACK_LIB_DIR}/${CMAKE_SHARED_LIBRARY_PREFIX}arpack${CMAKE_SHARED_LIBRARY_SUFFIX})
      set(AUTOBUILDARPACK TRUE)
      if (CMAKE_COMPILER_IS_GNUCXX)
        set(GCC_COVERAGE_COMPILE_FLAGS "-fpermissive")
        set(permissiveFlagIndex "-1")
        if (CMAKE_CXX_FLAGS)
          string(FIND ${CMAKE_CXX_FLAGS} ${GCC_COVERAGE_COMPILE_FLAGS} permissiveFlagIndex)
        endif()
        if (permissiveFlagIndex STREQUAL "-1")
          set(CMAKE_CXX_FLAGS "${CMAKE_CXX_FLAGS} ${GCC_COVERAGE_COMPILE_FLAGS}" CACHE STRING "compile flags" FORCE)
        endif()
      endif()
    else()
      message("####################################")
      message("###### ARPACK is not found !")
      message("####################################")
    endif()
  endif()
else()
  message(STATUS "ARPACK is not used")
endif()
mark_as_advanced(XLIFEPP_ARPACK_LIB)

##### Eighth macro : UMFPACK -> XLIFEPP_WITH_UMFPACK or XLIFEPP_WITHOUT_UMFPACK
#     ------------------------ LAPACK and BLAS required --------------------------
# cache option : UMFPACK_INCLUDE_DIR, UMFPACK_LIB_DIR, XLIFEPP_ENABLE_UMFPACK
# Checking UmfPack package
set(XLIFEPP_SUITESPARSE_HOME_DIR ${SUITESPARSE_HOME_DIR_DEFAULT} CACHE PATH "Path to SuiteSparse home directory")
set(XLIFEPP_SUITESPARSE_CONFIG_DIR " " CACHE PATH "Path to SuiteSparse_config directory")
set(XLIFEPP_UMFPACK_INCLUDE_DIR " " CACHE PATH "Path to UMFPACK include directory")
set(XLIFEPP_UMFPACK_LIB_DIR " " CACHE PATH "Path to UMFPACK lib directory")
set(XLIFEPP_AMD_INCLUDE_DIR " " CACHE PATH "Path to AMD include directory")
set(XLIFEPP_AMD_LIB_DIR " " CACHE PATH "Path to AMD lib directory")
set(XLIFEPP_CHOLMOD_INCLUDE_DIR " " CACHE PATH "Path to CHOLMOD include directory")
set(XLIFEPP_CHOLMOD_LIB_DIR " " CACHE PATH "Path to CHOLMOD lib directory")
set(XLIFEPP_CAMD_INCLUDE_DIR " " CACHE PATH "Path to CAMD include directory")
set(XLIFEPP_CAMD_LIB_DIR " " CACHE PATH "Path to CAMD lib directory")
set(XLIFEPP_COLAMD_INCLUDE_DIR " " CACHE PATH "Path to COLAMD include directory")
set(XLIFEPP_COLAMD_LIB_DIR " " CACHE PATH "Path to COLAMD lib directory")
set(XLIFEPP_CCOLAMD_INCLUDE_DIR " " CACHE PATH "Path to CCOLAMD include directory")
set(XLIFEPP_CCOLAMD_LIB_DIR " " CACHE PATH "Path to CCOLAMD lib directory")
set(XLIFEPP_METIS_INCLUDE_DIR " " CACHE PATH "Path to METIS include directory")
set(XLIFEPP_METIS_LIB_DIR " " CACHE PATH "Path to METIS lib directory")
set(XLIFEPP_SUITESPARSECONFIG_INCLUDE_DIR " " CACHE PATH "Path to SuiteSparse_config include directory")
set(XLIFEPP_SUITESPARSECONFIG_LIB_DIR " " CACHE PATH "Path to SuiteSparse_config lib directory")
if (APPLE)
  set(XLIFEPP_SUITESPARSE_LIB_DIR " " CACHE PATH "Path to SuiteSparse lib directory (Apple only)")
endif()
set(UMFPACK XLIFEPP_WITHOUT_UMFPACK)
if (XLIFEPP_ENABLE_UMFPACK)
  if (NOT LAPACK_FOUND)
    message("####################################")
    message("###### LAPACK and/or BLAS not found ! UMFPACK is not used")
    message("####################################")
  elseif (NOT BLAS_FOUND)
    message("####################################")
    message("###### LAPACK and/or BLAS not found ! UMFPACK is not used")
    message("####################################")
  else()
    # check if XLIFEPP_SUITESPARSE_HOME_DIR option is already in cache:
    xlifeppSetDefaultValue(SUITESPARSE_DIR XLIFEPP_SUITESPARSE_HOME_DIR ${SUITESPARSE_HOME_DIR_DEFAULT})

    # set default values for each option according to emptiness of XLIFEPP_SUITESPARSE_HOME_DIR
    set(DEFAULTVALUE_SUITESPARSECONFIG_DIR " ")
    set(DEFAULTVALUE_UMFPACK_INCLUDE_DIR " ")
    set(DEFAULTVALUE_UMFPACK_LIB_DIR " ")
    set(DEFAULTVALUE_AMD_INCLUDE_DIR " ")
    set(DEFAULTVALUE_AMD_LIB_DIR " ")
    set(DEFAULTVALUE_CHOLMOD_INCLUDE_DIR " ")
    set(DEFAULTVALUE_CHOLMOD_LIB_DIR " ")
    set(DEFAULTVALUE_CAMD_INCLUDE_DIR " ")
    set(DEFAULTVALUE_CAMD_LIB_DIR " ")
    set(DEFAULTVALUE_COLAMD_INCLUDE_DIR " ")
    set(DEFAULTVALUE_COLAMD_LIB_DIR " ")
    set(DEFAULTVALUE_CCOLAMD_INCLUDE_DIR " ")
    set(DEFAULTVALUE_CCOLAMD_LIB_DIR " ")
    set(DEFAULTVALUE_METIS_INCLUDE_DIR " ")
    set(DEFAULTVALUE_METIS_LIB_DIR " ")
    set(DEFAULTVALUE_SUITESPARSECONFIG_INCLUDE_DIR " ")
    set(DEFAULTVALUE_SUITESPARSECONFIG_LIB_DIR " ")
    if (APPLE)
      set(DEFAULTVALUE_SUITESPARSE_LIB_DIR " ")
    endif()
    if (NOT SUITESPARSE_DIR STREQUAL " ")
      set(DEFAULTVALUE_SUITESPARSECONFIG_DIR ${SUITESPARSE_DIR}/SuiteSparse_config)
      set(DEFAULTVALUE_UMFPACK_INCLUDE_DIR ${SUITESPARSE_DIR}/UMFPACK)
      set(DEFAULTVALUE_UMFPACK_LIB_DIR ${SUITESPARSE_DIR}/UMFPACK)
      set(DEFAULTVALUE_AMD_INCLUDE_DIR ${SUITESPARSE_DIR}/AMD)
      set(DEFAULTVALUE_AMD_LIB_DIR ${SUITESPARSE_DIR}/AMD)
      set(DEFAULTVALUE_CHOLMOD_INCLUDE_DIR ${SUITESPARSE_DIR}/CHOLMOD)
      set(DEFAULTVALUE_CHOLMOD_LIB_DIR ${SUITESPARSE_DIR}/CHOLMOD)
      set(DEFAULTVALUE_CAMD_INCLUDE_DIR ${SUITESPARSE_DIR}/CAMD)
      set(DEFAULTVALUE_CAMD_LIB_DIR ${SUITESPARSE_DIR}/CAMD)
      set(DEFAULTVALUE_COLAMD_INCLUDE_DIR ${SUITESPARSE_DIR}/COLAMD)
      set(DEFAULTVALUE_COLAMD_LIB_DIR ${SUITESPARSE_DIR}/COLAMD)
      set(DEFAULTVALUE_CCOLAMD_INCLUDE_DIR ${SUITESPARSE_DIR}/CCOLAMD)
      set(DEFAULTVALUE_CCOLAMD_LIB_DIR ${SUITESPARSE_DIR}/CCOLAMD)
      set(DEFAULTVALUE_METIS_INCLUDE_DIR ${SUITESPARSE_DIR}/metis-5.1.0)
      set(DEFAULTVALUE_METIS_LIB_DIR ${SUITESPARSE_DIR}/metis-5.1.0)
      set(DEFAULTVALUE_SUITESPARSECONFIG_INCLUDE_DIR ${SUITESPARSE_DIR}/SuiteSparse_config)
      set(DEFAULTVALUE_SUITESPARSECONFIG_LIB_DIR ${SUITESPARSE_DIR}/SuiteSparse_config)
      if(APPLE)
        set(DEFAULTVALUE_SUITESPARSE_LIB_DIR ${SUITESPARSE_DIR})
      endif()
    endif()

    # check if other options are already in cache:
    xlifeppSetDefaultValue(SUITESPARSECONFIG_DIR XLIFEPP_SUITESPARSE_CONFIG_DIR ${DEFAULTVALUE_SUITESPARSECONFIG_DIR})
    xlifeppSetDefaultValue(UMFPACK_INCLUDE_DIR XLIFEPP_UMFPACK_INCLUDE_DIR ${DEFAULTVALUE_UMFPACK_INCLUDE_DIR})
    xlifeppSetDefaultValue(UMFPACK_LIB_DIR XLIFEPP_UMFPACK_LIB_DIR ${DEFAULTVALUE_UMFPACK_LIB_DIR})
    xlifeppSetDefaultValue(AMD_INCLUDE_DIR XLIFEPP_AMD_INCLUDE_DIR ${DEFAULTVALUE_AMD_INCLUDE_DIR})
    xlifeppSetDefaultValue(AMD_LIB_DIR XLIFEPP_AMD_LIB_DIR ${DEFAULTVALUE_AMD_LIB_DIR})
    xlifeppSetDefaultValue(CHOLMOD_INCLUDE_DIR XLIFEPP_CHOLMOD_INCLUDE_DIR ${DEFAULTVALUE_CHOLMOD_INCLUDE_DIR})
    xlifeppSetDefaultValue(CHOLMOD_LIB_DIR XLIFEPP_CHOLMOD_LIB_DIR ${DEFAULTVALUE_CHOLMOD_LIB_DIR})
    xlifeppSetDefaultValue(CAMD_INCLUDE_DIR XLIFEPP_CAMD_INCLUDE_DIR ${DEFAULTVALUE_CAMD_INCLUDE_DIR})
    xlifeppSetDefaultValue(CAMD_LIB_DIR XLIFEPP_CAMD_LIB_DIR ${DEFAULTVALUE_CAMD_LIB_DIR})
    xlifeppSetDefaultValue(COLAMD_INCLUDE_DIR XLIFEPP_COLAMD_INCLUDE_DIR ${DEFAULTVALUE_COLAMD_INCLUDE_DIR})
    xlifeppSetDefaultValue(COLAMD_LIB_DIR XLIFEPP_COLAMD_LIB_DIR ${DEFAULTVALUE_COLAMD_LIB_DIR})
    xlifeppSetDefaultValue(CCOLAMD_INCLUDE_DIR XLIFEPP_CCOLAMD_INCLUDE_DIR ${DEFAULTVALUE_CCOLAMD_INCLUDE_DIR})
    xlifeppSetDefaultValue(CCOLAMD_LIB_DIR XLIFEPP_CCOLAMD_LIB_DIR ${DEFAULTVALUE_CCOLAMD_LIB_DIR})
    xlifeppSetDefaultValue(METIS_INCLUDE_DIR XLIFEPP_METIS_INCLUDE_DIR ${DEFAULTVALUE_METIS_INCLUDE_DIR})
    xlifeppSetDefaultValue(METIS_LIB_DIR XLIFEPP_METIS_LIB_DIR ${DEFAULTVALUE_METIS_LIB_DIR})
    xlifeppSetDefaultValue(SUITESPARSECONFIG_INCLUDE_DIR XLIFEPP_SUITESPARSECONFIG_INCLUDE_DIR ${DEFAULTVALUE_SUITESPARSECONFIG_INCLUDE_DIR})
    xlifeppSetDefaultValue(SUITESPARSECONFIG_LIB_DIR XLIFEPP_SUITESPARSECONFIG_LIB_DIR ${DEFAULTVALUE_SUITESPARSECONFIG_LIB_DIR})
    if (APPLE)
      xlifeppSetDefaultValue(SUITESPARSE_LIB_DIR XLIFEPP_SUITESPARSE_LIB_DIR ${DEFAULTVALUE_SUITESPARSE_LIB_DIR})
    endif()
    find_package(UMFPACK)
    if (UMFPACK_FOUND)
      include_directories(${UMFPACK_INCLUDES})
      set(UMFPACK XLIFEPP_WITH_UMFPACK)
      set(XLIFEPP_ENABLE_UMFPACK TRUE)
      set(XLIFEPP_UMFPACK_INCLUDE_DIR ${UMFPACK_INCLUDES})
      if (UMFPACK_LIBRARY)
        get_filename_component(XLIFEPP_UMFPACK_LIB_DIR ${UMFPACK_LIBRARY} PATH)
      endif()
      set(XLIFEPP_AMD_INCLUDE_DIR ${AMD_INCLUDE})
      if (AMD_LIBRARY)
        get_filename_component(XLIFEPP_AMD_LIB_DIR ${AMD_LIBRARY} PATH)
      endif()
      set(XLIFEPP_CHOLMOD_INCLUDE_DIR ${CHOLMOD_INCLUDE})
      if (CHOLMOD_LIBRARY)
        get_filename_component(XLIFEPP_CHOLMOD_LIB_DIR ${CHOLMOD_LIBRARY} PATH)
      endif()
      set(XLIFEPP_CAMD_INCLUDE_DIR ${CAMD_INCLUDE})
      if (CAMD_LIBRARY)
        get_filename_component(XLIFEPP_CAMD_LIB_DIR ${CAMD_LIBRARY} PATH)
      endif()
      set(XLIFEPP_COLAMD_INCLUDE_DIR ${COLAMD_INCLUDE})
      if (COLAMD_LIBRARY)
        get_filename_component(XLIFEPP_COLAMD_LIB_DIR ${COLAMD_LIBRARY} PATH)
      endif()
      set(XLIFEPP_CCOLAMD_INCLUDE_DIR ${CCOLAMD_INCLUDE})
      if (CCOLAMD_LIBRARY)
        get_filename_component(XLIFEPP_CCOLAMD_LIB_DIR ${CCOLAMD_LIBRARY} PATH)
      endif()
      set(XLIFEPP_METIS_INCLUDE_DIR ${METIS_INCLUDE})
      if (METIS_LIBRARY)
        get_filename_component(XLIFEPP_METIS_LIB_DIR ${METIS_LIBRARY} PATH)
      endif()
      set(XLIFEPP_SUITESPARSECONFIG_INCLUDE_DIR ${SUITESPARSECONFIG_INCLUDE})
      if (SUITESPARSECONFIG_LIBRARY)
        get_filename_component(XLIFEPP_SUITESPARSECONFIG_LIB_DIR ${SUITESPARSECONFIG_LIBRARY} PATH)
      endif()
      if (APPLE)
        if (SUITESPARSE_LIBRARY)
          get_filename_component(XLIFEPP_SUITESPARSE_LIB_DIR ${SUITESPARSE_LIBRARY} PATH)
        endif()
      endif()
      message(STATUS "UMFPACK is used")
    else()
      message("####################################")
      message("###### UMFPACK is not found !")
      message("####################################")
    endif()
  endif()
else()
  message(STATUS "UMFPACK is not used")
endif()

mark_as_advanced(XLIFEPP_SUITESPARSE_CONFIG_DIR
                 XLIFEPP_AMD_INCLUDE_DIR XLIFEPP_AMD_LIB_DIR
                 XLIFEPP_CHOLMOD_INCLUDE_DIR XLIFEPP_CHOLMOD_LIB_DIR
                 XLIFEPP_CAMD_INCLUDE_DIR XLIFEPP_CAMD_LIB_DIR
                 XLIFEPP_COLAMD_INCLUDE_DIR XLIFEPP_COLAMD_LIB_DIR
                 XLIFEPP_CCOLAMD_INCLUDE_DIR XLIFEPP_CCOLAMD_LIB_DIR
                 XLIFEPP_METIS_INCLUDE_DIR XLIFEPP_METIS_LIB_DIR
                 XLIFEPP_SUITESPARSECONFIG_INCLUDE_DIR XLIFEPP_SUITESPARSECONFIG_LIB_DIR)
if (APPLE)
  mark_as_advanced(XLIFEPP_SUITESPARSE_LIB_DIR)
endif()

##### Tenth macro : GMSH -> XLIFEPP_WITH_GMSH or XLIFEPP_WITHOUT_GMSH
# checking gmsh executable in PATH environment variable
set(GMSH_ADDITIONAL_PATHS "")
if (APPLE)
  set(GMSH_ADDITIONAL_PATHS /Applications/Gmsh.app/Contents/MacOS/)
endif()
if (WIN32)
  list(APPEND GMSH_ADDITIONAL_PATHS "C:/Program Files/gmsh")
  list(APPEND GMSH_ADDITIONAL_PATHS "C:/Program Files (x86)/gmsh")
endif()
if (NOT XLIFEPP_GMSH_EXECUTABLE)
  find_program(XLIFEPP_GMSH_EXECUTABLE gmsh PATHS $ENV{PATH} ${GMSH_ADDITIONAL_PATHS} DOC "Path to Gmsh executable" )
endif()
set(GMSH XLIFEPP_WITHOUT_GMSH)
if (${XLIFEPP_GMSH_EXECUTABLE} STREQUAL XLIFEPP_GMSH_EXECUTABLE_NOTFOUND OR ${XLIFEPP_GMSH_EXECUTABLE} STREQUAL XLIFEPP_GMSH_EXECUTABLE-NOTFOUND)
  message("####################################")
  message("###### GMSH is not found ! You should set the XLIFEPP_GMSH_EXECUTABLE to fix this")
  message("####################################")
else()
  get_filename_component(XLIFEPP_GMSH_EXECUTABLE ${XLIFEPP_GMSH_EXECUTABLE} REALPATH)
  get_filename_component(XLIFEPP_GMSH_DIR ${XLIFEPP_GMSH_EXECUTABLE} DIRECTORY)
  if (XLIFEPP_GMSH_DIR)
    set(GMSH XLIFEPP_WITH_GMSH)
    message(STATUS "Found GMSH : " ${XLIFEPP_GMSH_EXECUTABLE})
  else()
    message(STATUS "GMSH is not used")
  endif()
endif()

##### Eleventh macro : PARAVIEW -> XLIFEPP_WITH_PARAVIEW or XLIFEPP_WITHOUT_PARAVIEW
# checking gmsh executable in PATH environment variable
set(PARAVIEW_ADDITIONAL_PATHS  "")
if (APPLE)
  set(PARAVIEW_ADDITIONAL_PATHS /Applications/paraview.app/Contents/MacOS/)
endif()
if (WIN32)
  list(APPEND PARAVIEW_ADDITIONAL_PATHS "C:/Program Files/ParaView/bin")
  list(APPEND PARAVIEW_ADDITIONAL_PATHS "C:/Program Files (x86)/ParaView/bin")
endif()
set(PARAVIEW XLIFEPP_WITHOUT_PARAVIEW)
if (NOT XLIFEPP_PARAVIEW_EXECUTABLE)
  find_program(XLIFEPP_PARAVIEW_EXECUTABLE paraview PATHS $ENV{PATH} ${PARAVIEW_ADDITIONAL_PATHS} DOC "Path to Paraview executable")
endif()
if (${XLIFEPP_PARAVIEW_EXECUTABLE} STREQUAL XLIFEPP_PARAVIEW_EXECUTABLE_NOTFOUND OR ${XLIFEPP_PARAVIEW_EXECUTABLE} STREQUAL XLIFEPP_PARAVIEW_EXECUTABLE-NOTFOUND)
  message("####################################")
  message("###### Paraview is not found ! It is not used")
  message("####################################")
else()
  get_filename_component(XLIFEPP_PARAVIEW_EXECUTABLE ${XLIFEPP_PARAVIEW_EXECUTABLE} REALPATH)
  get_filename_component( XLIFEPP_PARAVIEW_DIR ${XLIFEPP_PARAVIEW_EXECUTABLE} DIRECTORY)
  if (XLIFEPP_PARAVIEW_DIR)
    set(PARAVIEW XLIFEPP_WITH_PARAVIEW)
    message(STATUS "Found Paraview : " ${XLIFEPP_PARAVIEW_EXECUTABLE})
  else()
    message(STATUS "Paraview is not used")
  endif()
endif()

##### Twelvth macro : EIGEN -> XLIFEPP_WITH_EIGEN or XLIFEPP_WITHOUT_EIGEN
set(XLIFEPP_EIGEN_DIR ${CMAKE_SOURCE_DIR}/ext/Eigen/Eigen CACHE PATH "Path to Eigen directory")
set(EIGEN XLIFEPP_WITH_EIGEN)
if (NOT XLIFEPP_ENABLE_OMP)
  message(STATUS "Eigen needs OpenMP to be activated")
  set(XLIFEPP_ENABLE_EIGEN OFF)
endif()
if (XLIFEPP_ENABLE_EIGEN)
  if (${XLIFEPP_CXX_COMPILER} STREQUAL "g++-4.7")
    set(EIGEN XLIFEPP_WITHOUT_EIGEN)
    message("####################################")
    message("##### g++ 4.7 is not able to compile Eigen library ! Eigen is not used")
    message("####################################")
  else()
    include_directories(${XLIFEPP_EIGEN_DIR})
    message(STATUS "Eigen is used")
  endif()
else()
  set(EIGEN XLIFEPP_WITHOUT_EIGEN)
  message(STATUS "Eigen is not used")
endif()

##### Thirtieth macro : AMOS -> XLIFEPP_WITH_AMOS or XLIFEPP_WITHOUT_AMOS
set(XLIFEPP_AMOS_DIR ${CMAKE_SOURCE_DIR}/ext/Amos CACHE PATH "Path to Amos directory")
set(AMOS XLIFEPP_WITH_AMOS)
if (XLIFEPP_ENABLE_AMOS)
  if (XLIFEPP_FROM_SRCS)
    if (NOT EXISTS ${XLIFEPP_AMOS_LIB})
      enable_language(Fortran)
      if (CMAKE_SYSTEM_NAME MATCHES "Darwin")
        cmake_policy(SET CMP0042 NEW) # needed by a cmake warning because of compilation of amos library
      endif()
      set(FORTRAN_ENABLED TRUE)
      file(GLOB AMOS_SRCS ${XLIFEPP_AMOS_DIR}/*.f)
      add_library(amos STATIC ${AMOS_SRCS})
      message(STATUS "Amos will be compiled and used")
    else()
      message(STATUS "Amos will be used")
    endif()
  else()
    message(STATUS "Amos will be used")
  endif()
  set(XLIFEPP_ENABLE_GFORTRAN TRUE)
else()
  set(AMOS XLIFEPP_WITHOUT_AMOS)
  message(STATUS "Amos is not used")
endif()

###### post work ######

# append arpack library to the libexts list
# Lapack and blas are already appended
if (XLIFEPP_ENABLE_ARPACK)
  foreach(lib ${XLIFEPP_ARPACK_LIB})
    get_filename_component(realLib ${lib} REALPATH)
    list(APPEND libexts ${realLib})
  endforeach()
endif()

# append umfpack library to the libexts list
# Lapack and blas are already appended
if (XLIFEPP_ENABLE_UMFPACK)
  foreach (lib ${UMFPACK_LIBRARIES})
    get_filename_component(realLib ${lib} REALPATH)
    list(APPEND libexts ${realLib})
  endforeach()
endif()

# append lapack library to the libexts list
if (XLIFEPP_ENABLE_LAPACK)
  foreach (lib ${XLIFEPP_LAPACK_LIB})
    get_filename_component(realLib ${lib} REALPATH)
    list(APPEND libexts ${realLib})
  endforeach()
endif()

# append blas library to the libexts list
if (XLIFEPP_ENABLE_BLAS)
  foreach (lib ${XLIFEPP_BLAS_LIB})
    get_filename_component(realLib ${lib} REALPATH)
    list(APPEND libexts ${realLib})
  endforeach()
endif()

set(XLIFEPP_FORTRAN_LIB_DIR " " CACHE PATH "Path to FORTRAN lib directory")
set(XLIFEPP_FORTRAN_LIB " " CACHE FILEPATH "FORTRAN lib with full path")

if (XLIFEPP_ENABLE_GFORTRAN)
  if (${XLIFEPP_FORTRAN_LIB} STREQUAL " ")
    list(APPEND libexts "-lgfortran")
    if (NOT ${XLIFEPP_FORTRAN_LIB_DIR} STREQUAL " ")
      get_filename_component(XLIFEPP_FORTRAN_LIB_DIR ${XLIFEPP_FORTRAN_LIB_DIR} REALPATH)
      list(APPEND libexts "-L${XLIFEPP_FORTRAN_LIB_DIR}")
    endif()
  else()
    list(APPEND libexts "${XLIFEPP_FORTRAN_LIB}")
  endif()
endif()
