#include "xlife++.h"
using namespace xlifepp;

//data function
Vector<Real> f(const Point& P, Parameters& pa = defaultParameters)
{  Vector<Real> F(2,0.); F(2)=-0.005; return F;}

int main(int argc, char** argv)
{
  init(_lang=en); // mandatory initialization of xlifepp

 //mesh rectangle
  Rectangle rect(_center=Point(0.,0.), _xlength=20, _ylength=2, _nnodes=Numbers(50,5), _domain_name="Omega", _side_names=Strings("","","","Gamma"));
  Mesh mesh2d(rect, triangle, 1, gmsh);
  Domain omega=mesh2d.domain("Omega"), Gamma=mesh2d.domain("Gamma");
  // create P1 Lagrange interpolation
  Space V(omega, P1, "V");
  Unknown u(V, "u", 2);  TestFunction v(u, "v");
  // create bilinear form and linear form
  Real lambda=112.134, mu=83.53, omg2=0, rho=7.86;
  BilinearForm auv = lambda*intg(omega, epsilon(u) % epsilon(v))
                   + 2*mu*intg(omega, div(u)*div(v)) - omg2*intg(omega,u|v);
  LinearForm fv=intg(omega, f|v);
  EssentialConditions ecs= (u|Gamma=0);
  TermMatrix A(auv, ecs,"A");
  TermVector B(fv, "B");
  //solve linear system AX=B using direct method
  TermVector U=directSolve(A, B);
  thePrintStream<<U;
  saveToFile("U", U, vtu);

  //create the deformation of the mesh
  for(number_t i=0;i<mesh2d.nbOfNodes();i++)
      mesh2d.nodes[i] += U.evaluate(mesh2d.nodes[i]).value<std::vector<Real> >();
  mesh2d.saveToFile("Ud",msh);

  return 0;
}
