#include "xlife++.h"
using namespace xlifepp;

int main() {
  using std::cout;
  using std::endl;
  init(); // mandatory initialization of XLiFE++

  cout << " Eigenvalues of the 1D Laplace operator with Neumann conditions\n";
  cout << " ==============================================================\n";

  int nbint=1; // number of intervalls
  int dk=60;   // interpolation degree
  cout << "Interpolation degree = " << dk << endl;

  // mesh : segment [0, pi]
  Mesh  zeroPi(Segment(_xmin=0, _xmax=pi_, _nnodes=nbint+1), 1);
  Domain omega = zeroPi.domain("Omega");

  Interpolation&  interp=interpolation(Lagrange, GaussLobatto, dk, H1);
  Space Vk(omega, interp, "Vk");
  Unknown u(Vk, "u");
  TestFunction v(u, "v");

  int qrodeg = 2*dk+1;
  BilinearForm muv = intg(omega, u * v, defaultQuadrature,qrodeg),
               auv = intg(omega, grad(u) | grad(v), defaultQuadrature,qrodeg);
  // Compute the Stiffness and Mass matrices
  TermMatrix S(auv,"auv"), M(muv,"muv");

  // The eigenvalue problem writes S x = l M x
  // Compute the nev first eigenvalues with smallest magnitude with Arpack
  int nev = 8;
  EigenElements areigs = arpackSolve(S,M, _nev=nev, _which="SM");

  cout << arEigInfos();

  cout.precision(17);
  cout << "Eigenvalues :" << endl;
  int nconv = areigs.numberOfEigenValues();
  for (int i = 0; i < nconv; i++) { cout << areigs.value(i+1).real() << endl; }

  saveToFile("Sy", areigs, matlab);
}
