#include "xlife++.h"
using namespace xlifepp;

Real f(const Point& P, Parameters& pa = defaultParameters)
{ Real x=P(1), y=P(2);
  return 32*(x*(1-x)+y*(1-y));}

int main(int argc, char** argv)
{
  init(_lang=en);
  //mesh square
  Mesh mesh2d(Square(_origin=Point(0.,0.), _length=1, _nnodes=21), triangle, 1, structured);
  Domain omega=mesh2d.domain("Omega");
  //create approximation P0 and RT1
  Space H(omega, P0, "H", false);
  Space V(omega, RaviartThomas, 1, "V", true);
  Unknown p(V,"p");
  TestFunction q(p,"q");  //p=grad(u)
  Unknown u(H,"u");
  TestFunction v(u,"v");
  //create problem (Poisson problem)
  TermMatrix A(intg(omega, p|q) + intg(omega, u*div(q)) - intg(omega, div(p)*v));
  TermVector b(intg(omega, f*v));
  //solve and save solution
  TermVector X=directSolve(A, b);
  saveToFile("u", X(u), vtu);
  return 0;
}
