#include "xlife++.h"
using namespace xlifepp;

Real f(const Point& P, Parameters& pa = defaultParameters)
{return -8.;}

int main(int argc, char** argv)
{
  init(_lang=en); // mandatory initialization of xlifepp

  //create mesh of square
  Strings sn("y=0","x=1","y=1","x=0");
  Square sq(_origin=Point(0.,0.), _length=1, _nnodes=20, _domain_name="Omega", _side_names=sn);
  Mesh mesh2d(sq, triangle, 1, structured);
  Domain omega=mesh2d.domain("Omega");
  Domain sigmaM=mesh2d.domain("x=0"),  sigmaP=mesh2d.domain("x=1");

  // create interpolation
  Space V(omega, P1, "V", true);
  Unknown u(V, "u");
  TestFunction v(u, "v");

  // create bilinear form, linear form and their algebraic representation
  BilinearForm auv=intg(omega, grad(u)|grad(v));
  LinearForm fv=intg(omega, f*v);
  EssentialConditions ecs= (u|sigmaM = 1) & (u|sigmaP = 1);
  TermMatrix A(auv, ecs, "A");
  TermVector B(fv, "B");

  // solve linear system AX=B
  TermVector U=directSolve(A, B);
  saveToFile("U_LD", U, vtu);
  return 0;
}
