#include "xlife++.h"
using namespace xlifepp;

Real cosx2(const Point& P, Parameters& pa = defaultParameters)
{
  Real x=P(1), y=P(2);
  return cos(pi_ * x) * cos(pi_ * y);
}

int main(int argc, char** argv)
{
  init(_lang=en);

  //mesh square
  Square sq(_origin=Point(0.,0.), _length=1, _nnodes=21);
  Mesh mesh2d(sq, triangle, 1, structured);
  Domain omega = mesh2d.domain("Omega");

  //build space and unknown
  FEInterpolation Pk=interpolation(Lagrange, standard, 1, H1);
  Space Vk(omega, Pk, "Vk", true);
  Unknown u(Vk, "u");
  TestFunction v(u, "v");

  // define variational formulation
  BilinearForm auv = intg(omega, grad(u) | grad(v)) + intg(omega, u * v);
  LinearForm fv=intg(omega, cosx2 * v);

  //compute matrix and right hand side
  TermMatrix A(auv, "a(u,v)");
  TermVector B(fv, "f(v)");

  // LLt factorize and solve
  TermMatrix LD;
  ldltFactorize(A, LD);
  TermVector U = factSolve(LD, B);

  saveToFile("U_LN", U, vtu);
  return 0;
}
