#include "xlife++.h"
using namespace xlifepp;

int main(int argc, char** argv)
{
  init(_lang=en); // mandatory initialization of xlifepp

   //mesh square
  Mesh mesh2d(Square(_origin=Point(0.,0.), _length=1, _nnodes=20), triangle, 1, gmsh);
  Domain omega = mesh2d.domain("Omega");

  //build  P2 interpolation
  Space Vk(omega, P2, "Vk", true);
  Unknown u(Vk, "u");
  TestFunction v(u, "v");

  //build eigen system
  BilinearForm auv = intg(omega, grad(u) | grad(v)) + intg(omega, u * v) ,
               muv = intg(omega, u * v);
  TermMatrix A(auv, "auv"), M(muv, "muv");

  //compute the 10 first smallest in magnitude
  EigenElements eigs = eigenInternSolve(A, M, _nev=10, _mode=_krylovSchur, _which="SM");   //internal solver
  thePrintStream << eigs.values;
  saveToFile("eigs", eigs.vectors, vtu);
  return 0;
}
